open Core.Std

(* The main function, called with the input channel *)
let main ~verbose ch =
  if ch = stdin then print_endline "Taking input from stdin.";
  let lexbuf = Lexing.from_channel ch in
  let (sch,cstr) = Parser.main Lexer.token lexbuf in
  if verbose then Schema.print sch;
  if Schema.is_regular sch then 
    print_endline (if Satisfiability.unsatisfiable (Regstab.regstab ~cstr sch) then "unsatisfiable" else "satisfiable")
  else raise Schema.Non_regular;
  print_newline ();
  flush stdout

(* Handling of command-line options, calls main *)
let _ =
  let channel = ref None in
  let verbose = ref false in
  let options = [
    "-verbose", Arg.Set(verbose), "Be verbose (i.e. currently, print the input schema after parsing)"
  ] in
  let usage = "regstab [file]    - says if the schema in file is satisfiable, schema taken on stdin if no file provided" in
  let file s = channel := Some(s) in
  Arg.parse options file usage;
  match !channel with
  |None      -> main stdin ~verbose:(!verbose)
  |Some file -> read_wrap ~f:(main ~verbose:(!verbose)) file

