(** Substitutions interface.
 * An interface to standardize the various substitution functions occurring all over the place:
 * substitution of an index, of a literal, schema or block.
 *
 * @author Vincent Aravantinos
 *)

module type Substituable =
  sig
    (** The type of variables. *)
    type variable

    (** The type of terms, i.e. of things we want to substitute variables with. *)
    type term

    (** The type of a substituable object, would it be an index, a schema or other. *)
    type substituable

    (** The substitution function. Substitutes [var] by [by] in a substituable data. *)
    val substitute : substituable -> var:variable -> by:term -> substituable
  end

